/*
 * imtim.c - Table-based Input Method for GTK+ 2 module.
 *
    Copyright (C) 2002-2003  Yao Zhang

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 2 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * yaoz@users.sourceforge.net
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <gtk/gtkimmodule.h>

#include "gtkimcontexttim.h"
#include "timtable.h"

#undef DEBUG

static GPtrArray *INFO_ARRAY;

void im_module_init(GTypeModule *type_module)
{
#ifdef DEBUG
fprintf(stderr, "im_module_init()\n");
#endif
    timtable_register_type(type_module);
    gtkimcontexttim_register_type(type_module);
}

void im_module_exit(void)
{
#ifdef DEBUG
fprintf(stderr, "im_module_exit()\n");
#endif
    if (INFO_ARRAY) {
	register int i;
	GtkIMContextInfo *info;

	for (i = 0; i < INFO_ARRAY->len; i++) {
	    info = g_ptr_array_index(INFO_ARRAY, i);
	    if (info != NULL) {
		free((void *)info->context_id);
		free((void *)info->context_name);
		free((void *)info->domain);
		free((void *)info->domain_dirname);
		free((void *)info->default_locales);
	    }
	}
        g_ptr_array_free(INFO_ARRAY, TRUE);
	INFO_ARRAY = NULL;
    }
}

void im_module_list(const GtkIMContextInfo ***contexts,
                    int *n_contexts)
{
#ifdef DEBUG
fprintf(stderr, "im_module_list()\n");
#endif
    if (INFO_ARRAY == NULL) {
	int i;
	GtkIMContextInfo *info;

        INFO_ARRAY = g_ptr_array_new();

        info = g_new(GtkIMContextInfo, 1);
	info->context_id = strdup("table.tim");
	info->context_name = strdup("Table-based Input Method");
	info->domain = strdup("gtk+");
	info->domain_dirname = strdup("");
	info->default_locales = strdup("");

	g_ptr_array_add(INFO_ARRAY, info);

	for (i = 0; i < 2; i++) {
	    GString *path;
	    GDir *dir;

            path = g_string_new("");
	    switch (i) {
	    case 0:
                g_string_append(path, getenv("HOME"));
                g_string_append(path, "/.wenju");
		break;
	    case 1:
                g_string_append(path, PKGDATADIR);
		break;
	    default:
		break;
	    }
	    if ((dir = g_dir_open(path->str, 0, NULL)) != NULL) {
	        const gchar *entry;

	        while ((entry = g_dir_read_name(dir)) != NULL) {
		    int len;

		    len = strlen(entry);
		    if ((len > 4) && (strcmp(entry+len-4, ".tim") == 0)) {
		        GString *name;

		        name = g_string_new("");
		        get_im_name(path->str, entry, name);
		        if (name->len > 0) {
			    g_string_append(name, " [TIM]");
        		    info = g_new(GtkIMContextInfo, 1);
			    info->context_id = strdup(entry);
			    info->context_name = strdup(name->str);
			    info->domain = strdup("gtk+");
			    info->domain_dirname = strdup("");
			    info->default_locales = strdup("");

			    g_ptr_array_add(INFO_ARRAY, info);
		        }
                        g_string_free(name, TRUE);
		    }
	        }
	        g_dir_close(dir);
	    }
            g_string_free(path, TRUE);
	}
    }

    *contexts = (const GtkIMContextInfo **)INFO_ARRAY->pdata;
    *n_contexts = INFO_ARRAY->len;
}

GtkIMContext *im_module_create(const gchar *context_id)
{
    GtkIMContextTIM *im;

#ifdef DEBUG
fprintf(stderr, "im_module_create()\n");
#endif
    if ((context_id == NULL) || (context_id[0] == '\0')) {
        return NULL;
    }

    im = g_object_new(GTKIMCONTEXTTIM_TYPE, NULL);
    GTKIMCONTEXTTIM_GET_CLASS(im)->set(im, context_id);

    return GTK_IM_CONTEXT(im);
}
